// -*- mode: C++; c-file-style: "Stroustrup" -*-
//
// This file is part of krot,
// a program for the simulation, assignment and fit of HRLIF spectra.
//
// Copyright (C) 1998 Jochen Kpper
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; see the file License. if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 , USA
//
// If you use this program for your scientific work, please cite it according to
// the file CITATION that comes with the package.
//
// $Id: dataset.h,v 1.7 1998/10/09 17:57:27 jochen Exp $


#ifndef KROT_DATASET_H
#define KROT_DATASET_H



#include "krot.h"
#include "transition.h"

#include <qcolor.h>
#include <qobject.h>
#include <qstring.h>

#include <kapp.h>

#include <sys/types.h>



class KRotData;


/**
 * This class provides the basic functionality common to all kinds of data sets
 * (like spectra / simulations).
 *
 * @short Storage object for data sets.
 * @author Jochen Kpper
 * @version $Id: dataset.h,v 1.7 1998/10/09 17:57:27 jochen Exp $
 */
class DataSet
{
public:

    enum { LINES_DAT_DATA, UNKNOWN_DATA, XY_DATA };

    /**
     * Constructor.
     */
    // DataSet();

    /**
     * Constructor.
     */
    DataSet( KRotData *krotdata );

    /**
     * Copy constructor.
     */
    DataSet( const DataSet& set );

    /**
     * Destructor; free all dynamically allocated memory.
     */
    virtual ~DataSet();


    /**
     * Compares the names of the two DataSets.
     *
     * @param set DataSet to compare to this.
     * @return True if this->name < set.name, false otherwise.
     */
    virtual bool operator<( const DataSet& set ) const;

    /**
     * Equality is defined as address identity (objetc identity) yet,
     * that might be changed when needed in some other context.
     *
     * @param set DataSet to compare to this.
     * @return True if the addresses of the two sets are equal, false otherwise.
     */
    virtual bool operator==( const DataSet& set ) const;

    /**
     * Read-only access to data points. This functions returns the 
     * intensity of this set at the frequency freq.
     *
     * @param freq Frequency to get the intensity at.
     * @return Averaged intensity.
     */
    virtual double operator[]( int64_t freq ) const;

    /**
     * On object identity nothing is done, otherwise all data from the given set
     * is copied to this.
     *
     * @param set DataSet to clone.
     * @return this.
     */
    virtual DataSet& operator=( const DataSet& set );

    /**
     * @return Color this dataset shall be painted in.
     */
    virtual const QColor& color() const;

    /**
     * The distance of two data points is fixed at 1 MHz at this time.
     *
     * @return Distance of two points.
     */
    virtual unsigned int delta() const;


    /**
     * Load dataset from file
     */
    virtual bool load( const QString& filename, int type );

    /**
     * @return Maximum intensity within this set.
     */
    virtual int64_t max() const;

    /**
     * @return Minimum intensity within this set.
     */
    virtual int64_t min() const;

    /**
     * @return Name of this dataset.
     *
     * @see setName
     * @see shortName
     */
    const QString& name() const;

    /**
     * @return Short name of this dataset. This is the pure filename
     * without any path.
     *
     * @see setName
     * @see name
     */
    const QString shortName() const;

    virtual bool save( const QString&, int ) const;

    /**
     * @return Intensity scaling factor.
     */
    virtual const double scaling() const;

    /**
     * @param color Set the color this set should be drawn in.
     */
    virtual void setColor( const QColor& color );

    /**
     * The name of any dataset should be the fully qualified filename, if this
     * set was imported from a file; If this set was generated by krot, this
     * is the name given to it by the user within krot.
     * 
     * @param name Name of this dataset.
     */
    virtual void setName( const QString& name );

    /**
     * Change the intensity scaling of this set.
     * 
     * @param scaling New intensity scaling factor.
     */
    virtual void setScaling( const double scaling );

    /**
     * @return Number of data points in this set.
     */
    virtual unsigned int size() const;

    /**
     * @return Start frequency of this set.
     */
    virtual int64_t start() const;

    /**
     * @return Stop frequency of this set.
     */
    virtual int64_t stop() const;


protected:

    virtual bool saveXY( const QString& ) const;


private:

    /**
     *
     */
    virtual bool loadLinesDat( const QString& );

    /**
     *
     */
    virtual bool loadXY( const QString& filename );


protected:

    /**
     * Color we shall be displayed in.
     */
    QColor clr;

    /**
     * intensities
     */
    int64_t *data;

    /**
     * frequency step width [MHz]
     */
    const u_int64_t dlta;

    KRotData *krotData;

    /**
     * maximum / minimum intensity
     */
    int64_t mx, mn;

    /**
     * name
     */
    QString nam;

    /**
     * user-adjustable y-shift of this dataset
     */
    int offset;

    /**
     * user-adjustable intensity-scaling
     */
    double scalg;

    /**
     * number of data points
     */
    u_int64_t siz;

    /**
     * first/last frequency
     */
    int64_t strt, stp;
};



#include "dataset_inline.h"



#endif
